<?php
//
//	Module name      :	dblog.php
//	Version          :	1.0
//	Started on       :	April 28, 2005
//	Completed on     :	April 28, 2005
//
//	Module type      :	Library
//	Description      :	Collection of functions to log data into fixed tables of a specified database
//						(via Odbc drivers, determined by a dsn, user and password)
//
//	Developed under  :	Php 4.1.2
//	Sw requirements  :	None
//
//	Parameters       :	None
//	Notes - usage    :	The database used must be composed by a table named "LogError" (see $DL_err_table_name
//						to change its name), to log errors from this module, with definition:
//							CREATE TABLE LogError (
//								Ts int(11) NOT NULL default '0',
//								ErrorMsg varchar(127) NOT NULL default '',
//								KEY Ts (Ts)
//							);
//						Other table must have as first field a Ts (key), followed by any number of string
//						fields with any index scheme; for instance:
//							CREATE TABLE InternalError (
//								Ts int(11) NOT NULL default '0',
//								Ip varchar(15) NOT NULL default '',
//								ErrorCode varchar(16) NOT NULL default '',
//								Url text NOT NULL,
//								KEY Ts (Ts),
//								KEY Ip (Ip)
//							);
//						The first field (Ts) is filled by this module, other fields are specified
//						by function parameters.
//


//--- Include files --------------------------------------------------------------------------------



//--- Public constants -----------------------------------------------------------------------------



define("DL_MODULE_NAME", "dblog.php 1.0; on April 28, 2005 by J.Hubb.");

// Error codes
define("DL_NO_ERROR",		0);
define("DL_CONNECT_ERROR",	1);
define("DL_INSERT_ERROR",	2);
define("DL_SELECT_ERROR",	3);
define("DL_HANDLE_ERROR",	4);
define("DL_LAST_ERROR",		4);



//--- Public variables -----------------------------------------------------------------------------



$DL_err_table_name = "LogError";



//--- Private constants ----------------------------------------------------------------------------



//--- Private variables ----------------------------------------------------------------------------



$_DL_db_handle = 0;
$_DL_odbc_msg = "";

$_DL_error_num = DL_NO_ERROR;
$_DL_error_msg = array(
	DL_NO_ERROR			=> "No error",
	DL_CONNECT_ERROR	=> &$_DL_odbc_msg,
	DL_INSERT_ERROR		=> &$_DL_odbc_msg,
	DL_SELECT_ERROR		=> &$_DL_odbc_msg,
	DL_HANDLE_ERROR		=> "Invalid db handle"
	);



//--- Public functions code ------------------------------------------------------------------------



function DLAppend($table)
// Logs a tuple to the table $table of the already initialized database, with data listed by parameters
// following $table; no check is performed on the number of following parameters; the insert query in
// the table is build according to parameters specified; all parameters are automatically escaped.
{
	global $_DL_db_handle, $_DL_error_num, $_DL_odbc_msg, $DL_err_table_name;

	$ts = time();
	$args = func_get_args();
	array_shift($args);
	foreach ($args as $n => $arg)
		$args[$n] = addslashes($arg);
	$args = implode("', '", $args);
	if (($resource = @odbc_exec($_DL_db_handle, "INSERT INTO $table VALUES ($ts, '$args')")) == FALSE)
	{
		if ($resource === FALSE)
		{
			$_DL_odbc_msg = odbc_errormsg();
			$_DL_error_num = DL_INSERT_ERROR;
		}
		else
			$_DL_error_num = DL_HANDLE_ERROR;
		@odbc_exec($_DL_db_handle,
				"INSERT INTO $DL_err_table_name VALUES ($ts, '" . addslashes($_DL_odbc_msg) . "')");
	}
	else
		$_DL_error_num = DL_NO_ERROR;
}



function DLDone()
// Closes the opened database connection.
{
	global $_DL_db_handle, $_DL_error_num, $_DL_odbc_msg;

	if (is_resource($_DL_db_handle))
	{
		odbc_close($_DL_db_handle);
		$_DL_db_handle = 0;
		$_DL_error_num = DL_NO_ERROR;
	}
	else
		$_DL_error_num = DL_HANDLE_ERROR;
}



function DLErrorMsg()
// Returns the error message of the last module function called.
{
	global $_DL_error_msg, $_DL_error_num;

	return $_DL_error_msg[$_DL_error_num];
}



function DLInit($db_dsn, $user, $pwd)
// Initializes the library, closing the previous database connection and opening a new one to
// the database Dsn $db_dsn, with user $user and password $pwd.
{
	global $_DL_db_handle, $_DL_error_num, $_DL_odbc_msg;

	if (($_DL_db_handle = @odbc_pconnect($db_dsn, $user, $pwd)) === FALSE)
	{
		$_DL_odbc_msg = odbc_errormsg();
		$_DL_error_num = DL_CONNECT_ERROR;
	}
	else
		$_DL_error_num = DL_NO_ERROR;
}



function DLResult()
// Returns the error code of the last module function called.
{
	global $_DL_error_num;

	return $_DL_error_num;
}



function DLSelectManyRows($query)
// Performs the query $query on database already initialized, returning the whole resulting tuples in
// an array of arrays. Use with caution, since this function could be very time and memory consuming!
// If the query fails, return FALSE.
{
	global $_DL_db_handle, $_DL_error_num, $_DL_odbc_msg, $DL_err_table_name;
	
	$res_array = array();
	if (($result = @odbc_exec($_DL_db_handle, $query)) === FALSE)
	{
		$_DL_odbc_msg = odbc_errormsg();
		@odbc_exec($_DL_db_handle, "INSERT INTO $DL_err_table_name VALUES (" . time() . ", '" .
				addslashes($_DL_odbc_msg) . "')");
		$_DL_error_num = DL_SELECT_ERROR;
		return FALSE;
	}
	while (@odbc_fetch_into($result, $row))
		array_push($res_array, $row);
	@odbc_free_result($result);
	$_DL_error_num = DL_NO_ERROR;
	return $res_array;
}



//--- Private functions code -----------------------------------------------------------------------



//--- Initialization code --------------------------------------------------------------------------



?>
